"""Recommended values related to security controls"""
from .utils import global_from_env, configure_authentication_backends

#  AC-7 Automatic lockout of users after invalid login attempts
AUTHENTICATION_BACKENDS = configure_authentication_backends([
    'django.contrib.auth.backends.ModelBackend',
])

# Lockout users based on their username and IP address
AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP = True

# Log unsuccessful logins
AXES_ENABLE_ACCESS_FAILURE_LOG = True

# After 3 failed login attempts, lock out the combination (IP + user).
AXES_FAILURE_LIMIT = 3

# After 30 minutes, locked out accounts are automatically unlocked.
AXES_COOLOFF_TIME = 0.5

# Reverse proxy configuration
AXES_PROXY_COUNT = 1  # (Behind 1 proxy)
AXES_META_PRECEDENCE_ORDER = [
    'HTTP_X_FORWARDED_FOR',
    'REMOTE_ADDR',
]

#  AC-11 - Session controls
global_from_env(
    # Sessions expire in 20 minutes
    SESSION_COOKIE_AGE=(int, 1200),
    # Use HTTPS for session cookie
    SESSION_COOKIE_SECURE=(bool, True),
    # Sessions close when browser is closed
    SESSION_EXPIRE_AT_BROWSER_CLOSE=(bool, True),
    # Every requests extends the session (This is required for the WET session
    # plugin to function properly.)
    SESSION_SAVE_EVERY_REQUEST=(bool, True),
)



"""
Decorators used to provide internationalization
"""
from modeltranslation.translator import translator, TranslationOptions

from phac_aspc.django.helpers.ready import execute_when_ready


class translate:  # pylint: disable=invalid-name,too-few-public-methods
    """Add localization to a model
    Usage:
        >>> from django.db import models
        >>> from phac_aspc.django.localization.decorators import translate
        >>>
        >>> @translate('title')
        >>> class Person(models.Model):
        >>>     name = models.CharField(max_length=255)
        >>>     title = models.CharField(max_length=255)

        The ``title`` field will now support multiple languages as defined in
        ``LANGUAGES``.

        Special care should be taken when adding a translated model to Django's
        admin interface.  For convenience you can use the add_admin decorator
        provided by this package, or see
        https://django-modeltranslation.readthedocs.io/en/latest/

    """

    def __init__(self, fields):
        self.fields = fields

    def __call__(self, cls):
        def register_model():
            translator.register(
                cls,
                type(
                    'f"{cls.__name__}TranslationOptions"',
                    (TranslationOptions,),
                    {"fields": self.fields},
                ),
            )
            cls.__is_translated_model__ = True

        execute_when_ready(register_model)
        return cls



"""Setup URLs for views related to WET"""
from django.urls import path

from ..views import wet

urlpatterns = [
    path("phac-aspc/helpers/session", wet.session, name="phac_aspc_helpers_session")
]



"""Unit tests for utils.py"""
from django.core.checks.registry import registry

from .utils import (
    trigger_configuration_warning,
    warn_and_remove,
    configure_apps,
    configure_authentication_backends,
    configure_middleware,
)


def test_trigger_configuration_warning():
    """Test the trigger configuration warning successfully registers its
    check function"""
    num = len(registry.get_checks())
    msg = "this is a warning"
    trigger_configuration_warning(msg)
    assert len(registry.get_checks()) == num + 1
    assert "phac_aspc_conf_warning" in [i.__name__ for i in registry.get_checks()]
    for i in registry.get_checks():
        if i.__name__ == "phac_aspc_conf_warning":
            warn = i(None)
            assert len(warn) == 1
            assert warn[0].msg == msg


def test_warn_and_remove():
    """Test that if an item appears in the list, it is removed from the result
    and a configuration warning is called."""
    num = len(registry.get_checks())
    test = warn_and_remove(["a", "b"], ["c"])
    assert len(registry.get_checks()) == num
    assert test == ["a", "b"]

    num = len(registry.get_checks())
    test = warn_and_remove(["a", "b", "c", "d"], ["c"])
    assert len(registry.get_checks()) == num + 1
    assert test == ["a", "b", "d"]

    num = len(registry.get_checks())
    test = warn_and_remove(["a", "b", "c", "d"], ["a", "b", "c"])
    assert len(registry.get_checks()) == num + 3
    assert test == ["d"]

    num = len(registry.get_checks())
    test = warn_and_remove(["a", "b", "c", "d"], [])
    assert len(registry.get_checks()) == num
    assert test == ["a", "b", "c", "d"]

    num = len(registry.get_checks())
    test = warn_and_remove([], [])
    assert len(registry.get_checks()) == num
    assert not test

    num = len(registry.get_checks())
    test = warn_and_remove(["a", "b", "c", "d"], ["a", "b", "c", "d"])
    assert len(registry.get_checks()) == num + 4
    assert not test


def test_configure_apps():
    """Test that the configure apps utility adds the correct apps"""
    num = len(registry.get_checks())
    test = configure_apps([])
    assert test == ["modeltranslation", "axes", "phac_aspc.django.helpers"]
    assert len(registry.get_checks()) == num

    num = len(registry.get_checks())
    test = configure_apps(["a", "b"])
    assert test == ["modeltranslation", "axes", "a", "b", "phac_aspc.django.helpers"]
    assert len(registry.get_checks()) == num

    num = len(registry.get_checks())
    test = configure_apps(["a", "axes", "b"])
    assert test == ["modeltranslation", "a", "axes", "b", "phac_aspc.django.helpers"]
    assert len(registry.get_checks()) == num + 1

    num = len(registry.get_checks())
    test = configure_apps(["phac_aspc.django.helpers", "axes", "b"])
    assert test == ["modeltranslation", "phac_aspc.django.helpers", "axes", "b"]
    assert len(registry.get_checks()) == num + 2


def test_configure_authentication_backends():
    """Test that the configure_authentication_backends utility adds the correct
    backends to the list"""
    num = len(registry.get_checks())
    test = configure_authentication_backends([])
    assert test == ["axes.backends.AxesStandaloneBackend"]
    assert len(registry.get_checks()) == num

    num = len(registry.get_checks())
    test = configure_authentication_backends(["a", "b"])
    assert test == ["axes.backends.AxesStandaloneBackend", "a", "b"]
    assert len(registry.get_checks()) == num

    num = len(registry.get_checks())
    test = configure_authentication_backends(
        ["a", "axes.backends.AxesStandaloneBackend", "b"]
    )
    assert test == ["a", "axes.backends.AxesStandaloneBackend", "b"]
    assert len(registry.get_checks()) == num + 1


def test_configure_middleware():
    """Test that the configure_middleware utility adds the correct middleware to
    the list"""
    num = len(registry.get_checks())
    test = configure_middleware([])
    assert test == ["axes.middleware.AxesMiddleware"]
    assert len(registry.get_checks()) == num

    num = len(registry.get_checks())
    test = configure_middleware(["a", "b"])
    assert test == ["axes.middleware.AxesMiddleware", "a", "b"]
    assert len(registry.get_checks()) == num

    num = len(registry.get_checks())
    test = configure_middleware(["a", "axes.middleware.AxesMiddleware", "b"])
    assert test == ["a", "axes.middleware.AxesMiddleware", "b"]
    assert len(registry.get_checks()) == num + 1



"""Related to implementing WET"""
from django import template
from django.utils.translation import get_language

register = template.Library()


@register.simple_tag()
def phac_aspc_localization_lang():
    """Returns the current language code"""
    return get_language()



"""Setup django for pytest tests"""

import django
from django.conf import settings

settings.configure(
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.contenttypes",
        "phac_aspc.django.helpers",
        "phac_aspc.django",
    ],
    USE_I18N=True,
    LANGUAGE=(
        ("fr-ca", "French"),
        ("en-ca", "English"),
    ),
)
django.setup()
